from setuptools import setup, find_packages
import os, sys

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()

install_requires = [
    # List your project dependencies here.
    # For more details, see:
    # http://packages.python.org/distribute/setuptools.html#declaring-dependencies
    'pygoo>=0.1.3',
    'pycurl',
    'cheetah',
    'lxml',
    'feedparser'
]


datafiles_exts = [ '*.png', '*.svg', '*.tmpl', '*.css', '*.html', '*.js' ]

# py2app data
APP = ['bin/smewg.py']
DATA_FILES = []
OPTIONS = { 'argv_emulation': True,
            'iconfile': 'smewt/icons/smewt.icns',
            'packages': [ 'smewt', 'Cheetah', 'lxml' ],
            'frameworks': [ '/Developer/Applications/Qt/plugins/iconengines/libqsvgicon.dylib' ],
            'includes': [ 'sip', 'PyQt4', 'PyQt4.QtCore', 'PyQt4.QtGui', 'PyQt4.QtNetwork', 'PyQt4.QtWebKit',  'PyQt4.QtXml', 'PyQt4.QtSvg' ],
            'excludes': [ 'PyQt4.QtDesigner', 'PyQt4.QtOpenGL', 'PyQt4.QtScript',
                          'PyQt4.QtSql', 'PyQt4.QtTest' ] # 'PyQt4.phonon'
            }


args = dict(name = 'smewt',
            version = '0.2',
            description = 'Smewt - a smart media manager.',
            long_description = README + '\n\n' + NEWS,
            classifiers = [], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
            keywords = 'smewt pygoo media manager video collection',
            author = 'Nicolas Wack',
            author_email = 'wackou@gmail.com',
            url = 'http://www.smewt.com/',
            license = 'GPLv3',
            packages = find_packages(exclude = [ 'ez_setup', 'examples', 'tests' ]),
            #package_data = dict((package, datafiles_exts) for package in find_packages()),
            include_package_data = True,
            scripts = [ 'bin/smewg' ],
            install_requires = install_requires
            )

if sys.platform == 'darwin':
    args.update(dict(# for py2app
                     name = 'Smewt',
                     app=APP,
                     data_files=DATA_FILES,
                     options={'py2app': OPTIONS, 'plist':dict(CFBundleIdentifier = 'com.smewt.Smewt')},
                     setup_requires=['py2app']
                     ))

setup(**args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2011 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, Media, Metadata
from smewt.solvers.solver import Solver


class MergeSolver(Solver):
    """A MergeSolver finds all nodes of the given type in a graph and merges their data into a single one.

    Each node should have a 'confidence' attribute that will be used for resolving conflicts. Only the value
    with the highest confidence is kept in the end."""

    def __init__(self, type):
        super(MergeSolver, self).__init__()
        self.type = type

    def perform(self, query):
        self.checkValid(query)

        results = sorted(query.find_all(node_type = self.type), key = lambda x: -x.confidence)

        # we have ownership over the query graph, so we can modify its elements
        result = results[0]
        for md in results[1:]:
            for k, v in md.explicit_items():
                if k == 'confidence' or k in result.keys():
                    continue
                result.set(k, v)

        return self.found(query, result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os, os.path, subprocess, logging
import threading, time

log = logging.getLogger('smewt.plugins.amulecommand')

class ProcessTimeout(threading.Thread):
    '''Class that checks that the given process doesn't run for longer than a given number of seconds.'''
    def __init__(self, process, maxSeconds = 5):
        super(ProcessTimeout, self).__init__()
        self.process = process
        self.maxSeconds = maxSeconds

    def run(self):
        secs = 0
        while self.process.poll() is None:
            time.sleep(1)
            secs += 1
            if secs >= self.maxSeconds:
                # need to kill process, starting from python 2.6 we can use process.terminate()
                # meanwhile, let's just kill it by hand
                subprocess.Popen(['kill', str(self.process.pid)])
                return


def recreateAmuleRemoteConf():
    amuleConf = os.path.join(os.environ['HOME'], '.aMule', 'amule.conf')
    amulecmd = os.path.join('/usr', 'bin', 'amulecmd')
    if os.path.exists(amuleConf) and os.path.exists(amulecmd):
        cmd = [ amulecmd, '--create-config-from=' + amuleConf ]
        subprocess.Popen(cmd)
    else:
        log.warning('AmuleCommand: you need amule and amulecmd to be installed for aMule integration...')
        log.warning('AmuleCommand: on Debian/Ubuntu systems you can find the amulecmd in the "amule-utils" package')

# try to regenerate remote.conf when we import that module
recreateAmuleRemoteConf()

class AmuleCommand():
    def __init__(self):
        # message by default if the amulecmd process gets killed
        self.errmsg = "aMule doesn't appear to be running..."

    def download(self, ed2kLink):
        cmd = [ 'amulecmd', '--command=Add %s' % ed2kLink ]
        amuleProc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        ProcessTimeout(amuleProc, maxSeconds = 5).start()
        amuleReply, _ = amuleProc.communicate()

        print 'amule said:', amuleReply
        errheader = ' > Request failed with the following error: '
        errmsg = None
        for l in amuleReply.split('\n'):
            if l.startswith(errheader):
                self.errmsg = l[len(errheader):].strip()

        success = '> Operation was successful.' in amuleReply
        if success:
            return (True, None)
        else:
            return (False, self.errmsg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewtdict import SmewtDict, ValidatingSmewtDict
from smewtexception import SmewtException
from smewturl import SmewtUrl
from solvingchain import SolvingChain
from cache import cachedmethod
from eventserver import EventServer
from mediaobject import Media, Metadata
from taskmanager import Task, TaskManager
from importtask import ImportTask
from subtitletask import SubtitleTask
from actionfactory import ActionFactory
from graphaction import GraphAction
from collection import Collection
from smewtdaemon import SmewtDaemon



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# string cleaning related functions

def stripBrackets(s):
    if not s:
        return s
    if s[0] == '[' and s[-1] == ']': return s[1:-1]
    if s[0] == '(' and s[-1] == ')': return s[1:-1]
    if s[0] == '{' and s[-1] == '}': return s[1:-1]
    return s


def cleanString(s):
    for c in ('.', '-', '_'):
        s = s.replace(c, ' ')
    parts = s.split()
    return ' '.join(p for p in parts if p != '')

# regexps-related functions
import re
from smewtexception import SmewtException
from PyQt4.QtCore import QString

def simpleMatch(string, regexp):
    try:
        return re.compile(regexp).search(string).groups()[0]
    except IndexError:
        raise SmewtException("'%s' Does not match regexp '%s'" % (string, regexp))

def matchRegexp(string, regexp):
    """Tries to match the given string against the regexp (using named match groups)
    and raises a SmewtException if it didn't match."""
    match = re.compile(regexp, re.IGNORECASE | re.DOTALL).search(string)
    if match:
        return match.groupdict()
    raise SmewtException("'%s' Does not match regexp '%s'" % (string, regexp))

def matchAllRegexp(string, regexps):
    """Matches the string against a list of regexps (using named match groups) and
    returns a list of all found matches."""
    result = []
    for regexp in regexps:
        s = string
        rexp = re.compile(regexp, re.IGNORECASE)
        match = rexp.search(s)
        while match:
            result.append(match.groupdict())
            s = s[match.span()[1]:]
            match = rexp.search(s)
    return result

def matchAnyRegexp(string, regexps):
    """Matches the string against a list of regexps (using named match groups) and
    returns the first match it could find, None if not found."""
    for regexp in regexps:
        result = re.compile(regexp, re.IGNORECASE).search(string)
        if result:
            return result.groupdict()
    return None

def multipleMatchRegexp(string, regexp):
    """Matches the given string against the regexp (using named match groups) and returns
    a list of all found matches in the string"""
    rexp = re.compile(regexp, re.IGNORECASE | re.DOTALL)
    result = []
    while True:
        match = rexp.search(string)
        if match:
            result += [ match.groupdict() ]
            # keep everything after what's been matched, and try to match again
            string = string[match.end(match.lastindex):]
        else:
            return result


#string-related functions

def between(s, left, right):
    return s.split(left)[1].split(right)[0]

def toUtf8(o):
    '''converts all unicode strings found in the given object to utf-8 strings'''
    if isinstance(o, QString):
        o = unicode(o)

    if isinstance(o, unicode):
        return o.encode('utf-8')
    elif isinstance(o, list):
        return [ toUtf8(i) for i in o ]
    elif isinstance(o, dict):
        result = {}
        for key, value in o.items():
            result[toUtf8(key)] = toUtf8(value)
        return result

    else:
        return o


def levenshtein(a, b):
    if not a: return len(b)
    if not b: return len(a)

    m = len(a)
    n = len(b)
    d = []
    for i in range(m+1):
        d.append([0] * (n+1))

    for i in range(m+1):
        d[i][0] = i

    for j in range(n+1):
        d[0][j] = j

    for i in range(1, m+1):
        for j in range(1, n+1):
            if a[i-1] == b[j-1]:
                cost = 0
            else:
                cost = 1

            d[i][j] = min(d[i-1][j] + 1,     # deletion
                          d[i][j-1] + 1,     # insertion
                          d[i-1][j-1] + cost # substitution
                          )

    return d[m][n]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pygoo import BaseObject, MemoryObjectGraph, Equal
from smewtdict import SmewtDict, ValidatingSmewtDict
from smewtexception import SmewtException
from textutils import toUtf8

# This file contains the 2 base MediaObject types used in Smewt:
#  - Media: is the type used to represent physical files on the hard disk.
#    It always has at least 2 properties: 'filename' and 'sha1'
#  - Metadata: is the type used to represent a media entity independent
#    of its physical location.
#
# Two MediaObject can point to the same AbstractMediaObject, such as the video and
# the subtitle files for an episode will point to the same Episode AbstractMediaObject
#
# The job of a guesser is to map a MediaObject to its corresponding AbstractMediaObject


class Metadata(BaseObject):
    schema = { 'confidence': float,
               'watched': bool
               }

    valid = []


class Media(BaseObject):
    schema = { 'filename': unicode,
               'sha1': unicode,
               'metadata': Metadata,
               'watched': bool, # TODO: or is the one from Metadata sufficient?
               'lastAccessed': float, # seconds since the epoch

               # used by guessers and solvers
               'matches': Metadata
               }

    valid = [ 'filename' ]
    unique = [ 'filename' ]
    reverse_lookup = { 'metadata': 'files',
                       'matches': 'query' }

    types = { 'video': [ 'avi', 'ogm', 'mkv', 'mpg', 'mpeg' ],
              'subtitle': [ 'idx', 'sub', 'srt' ]
              }


    def ext(self):
        return self.filename.split('.')[-1]

    def type(self):
        ext = self.ext().lower()
        for name, exts in Media.types.items():
            if ext in exts:
                return name
        return 'unknown type'


def foundMetadata(query, result, link = True):
    """Return a graph that contains:
     - the only Media object found in the query graph
     - the result object linked as metadata to the previous media object

    WARNING: this functions messes with the data in the query graph, do not reuse it after
    calling this function.
    """
    # TODO: check that result is valid
    solved = MemoryObjectGraph()

    # remove the stale 'matches' link before adding the media to the resulting graph
    #query.display_graph()
    media = query.find_one(Media)
    media.matches = []
    media.metadata = []
    m = solved.add_object(media)

    if result is None:
        return solved

    if isinstance(result, list):
        result = [ solved.add_object(n, recurse = Equal.OnLiterals) for n in result ]
    else:
        result = solved.add_object(result, recurse = Equal.OnLiterals)

    #solved.display_graph()
    if link:
        m.metadata = result

    return solved



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import Media

class Tagger(object):
    def __init__(self):
        super(Tagger, self).__init__()

    def cleanup(self, result):
        for o in result.find_all(Media):
            o.matches = []